/*
 * Copyright (c) 2023 DTB <trinity@trinity.moe>
 * SPDX-License-Identifier: AGPL-3.0-or-later
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

#include <stdio.h> /* fprintf(3), fputs(3), getc(3), putc(3), stdin, stdout,
                    * EOF */
#include <stdlib.h> /* EXIT_FAILURE, EXIT_SUCCESS */
#include <unistd.h> /* getopt(3) */
#include <sysexits.h>

int main(int argc, char *argv[]){
	int c;
	char showend;
	char showtab;

	showend = 0;
	showtab = 0;

	if(argc > 0)
		while((c = getopt(argc, argv, "et")) != -1)
			switch(c){
			case 'e':	showend = 1; break;
			case 't':	showtab = 1; break;
			default:	goto usage;
			}

	if(argc > optind){
usage:		fprintf(stderr, "Usage: %s [-et]\n", argv[0]);
		return EX_USAGE;
	}

	while((c = getc(stdin)) != EOF){
		if((c & 0x80) != 0)
			fputs("M-", stdout);
		switch(c ^ 0x80 /* 0b 1000 0000 */){
		case 0x7f:	fputs("^?", stdout);
				break;
		case '\n':	if(showend)
					putc('$', stdout);
		default:
			if(c >= ' ' || c == '\n' || (!showtab && c == '\t'))
				putc(c, stdout);
			else
				fprintf(stdout, "^%c", c + '@');
		}
	}

	return EX_OK;
}
